import { Construct, ISynthesisSession } from '@aws-cdk/core';
import { ApiObject } from './api-object';
export declare class Chart extends Construct {
    /**
     * Finds the chart in which a node is defined.
     * @param node a construct node
     */
    static of(node: Construct): Chart;
    /**
     * The name of the stack's YAML file as emitted into the cloud assembly
     * directory during synthesis.
     */
    readonly manifestFile: string;
    constructor(scope: Construct, ns: string);
    /**
     * Generates a app-unique name for an object given it's construct node path.
     *
     * Different resource types may have different constraints on names
     * (`metadata.name`). The previous version of the name generator was
     * compatible with DNS_SUBDOMAIN but not with DNS_LABEL.
     *
     * For example, `Deployment` names must comply with DNS_SUBDOMAIN while
     * `Service` names must comply with DNS_LABEL.
     *
     * Since there is no formal specification for this, the default name
     * generation scheme for kubernetes objects in cdk8s was changed to DNS_LABEL,
     * since it’s the common denominator for all kubernetes resources
     * (supposedly).
     *
     * You can override this method if you wish to customize object names at the
     * chart level.
     *
     * @param apiObject The API object to generate a name for.
     */
    generateObjectName(apiObject: ApiObject): string;
    protected synthesize(session: ISynthesisSession): void;
}
